=== modified file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	2006-03-03 09:10:46 +0000
+++ bzrlib/lockable_files.py	2006-03-06 04:09:20 +0000
@@ -84,7 +84,9 @@
             warn("LockableFiles: lock_class is now a mandatory parameter",
                  DeprecationWarning, stacklevel=2)
             lock_class = TransportLock
-        self._lock = lock_class(transport, esc_name)
+        self._lock = lock_class(transport, esc_name, 
+                                file_modebits=self._file_mode,
+                                dir_modebits=self._dir_mode)
 
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__,
@@ -267,9 +269,11 @@
     This is suitable for use only in WorkingTrees (which are at present
     always local).
     """
-    def __init__(self, transport, escaped_name):
+    def __init__(self, transport, escaped_name, file_modebits, dir_modebits):
         self._transport = transport
         self._escaped_name = escaped_name
+        self._file_modebits = file_modebits
+        self._dir_modebits = dir_modebits
 
     def lock_write(self):
         self._lock = self._transport.lock_write(self._escaped_name)
@@ -281,5 +285,7 @@
         self._lock.unlock()
         self._lock = None
 
-    # TODO: for old locks we have to manually create the file the first time
-    # it's used; this should be here too.
+    def create(self):
+        """Create lock mechanism"""
+        # for old-style locks, create the file now
+        self._transport.put(lock_file, StringIO(), mode=self._file_modebits)

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2006-03-06 03:57:18 +0000
+++ bzrlib/lockdir.py	2006-03-06 04:09:20 +0000
@@ -129,6 +129,9 @@
 # TODO: Some kind of callback run while polling a lock to show progress
 # indicators.
 
+# TODO: Make sure to pass the right file and directory mode bits to all
+# files/dirs created.
+
 _DEFAULT_TIMEOUT_SECONDS = 300
 _DEFAULT_POLL_SECONDS = 0.5
 
@@ -137,7 +140,7 @@
 
     __INFO_NAME = '/info'
 
-    def __init__(self, transport, path):
+    def __init__(self, transport, path, file_modebits=0644, dir_modebits=0755):
         """Create a new LockDir object.
 
         The LockDir is initially unlocked - this just creates the object.
@@ -155,6 +158,9 @@
         self._fake_read_lock = False
         self._held_dir = path + '/held'
         self._held_info_path = self._held_dir + self.__INFO_NAME
+        self._base_exists = False
+        self._file_modebits = file_modebits
+        self._dir_modebits = dir_modebits
         self.nonce = rand_chars(20)
 
     def __repr__(self):
@@ -173,10 +179,13 @@
         if self.transport.is_readonly():
             raise UnlockableTransport(self.transport)
         self.transport.mkdir(self.path)
+        self._base_exists = True
 
     def _create_if_needed(self):
         # XXX: remove this in favour of creating the lock when the containing
         # directory is built
+        if self._base_exists:
+            return
         if not self.transport.has(self.path):
             self.create()
 

